/*******************************************************************************/
/*                                                                             */
/*  iploop.c, adress calculations main loop                                    */
/*  Copyright (C) 2002 Dennis Pries                                            */
/*                                                                             */
/*  This program is free software; you can redistribute it and/or modify       */
/*  it under the terms of the GNU General Public License as published by       */
/*  the Free Software Foundation; either version 2 of the License, or          */
/*  (at your option) any later version.                                        */
/*                                                                             */
/*  This program is distributed in the hope that it will be useful,            */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/*  GNU General Public License for more details.                               */
/*                                                                             */
/*  You should have received a copy of the GNU General Public License          */
/*  along with this program; if not, write to the Free Software                */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */
/*                                                                             */
/*******************************************************************************/

#include	"pvipcalc.h"
#include	"ip.h"
#include	"gdata.h"
#include	"generic.h"
#include	"loops.h"
#include	"list.h"

int iploop(char *ip)
{
  int ret_code=0;
  TCHSTS tsts;
/*  char ip[32] = ""; */

  IP cip;
  IP network = {};
  IP broadcast = {};
  float addresses = 0, usableaddresses = 0;
  char totaladdr[12];
  char usableaddr[12];
  char snetwork[32];
  char sbroadcast[32];
  char sprefix[2];
  byte prefix = 0;
  word cursor = 0;
  int  err;
  bool dispinfo = FALSE;
  bool calcnow = FALSE;

  TXTP		EventPrm;
  TCHTBL	TchTxtBuf[3];
  T_SCR_POS	ComScr;
  word		GdTbl[1];

  int		funcIdx;
  char      funcName;
  char		KeyCode;

  GdTbl[0]		  = 0xffff;			/* No Guidance comment */
  EventPrm.st_x	  = 7;				/* Start coordinate (X) of text display	*/
  EventPrm.st_y	  = 30;				/* Start coordinate (Y) of text display	*/
  EventPrm.ed_x	  = 152;			/* End coordinate (X) of text display */
  EventPrm.it_y	  = 9;				/* Text display line spacing (Y) */
  EventPrm.MAXGYO = 1;				/* Number of text display lines	*/
  EventPrm.font   = IB_PFONT1;		/* Display font type */
  EventPrm.csen	  = TRUE;			/* Cursor display enabled (Keyword registration enabled) */
  EventPrm.rtnen  = 0x02;			/* CR code display disabled	*/
  EventPrm.maxmj  = 31;				/* Maximum number of allowable input characters. */
  EventPrm.txbf   = ip;   			/* Designation of text buffer address */
  EventPrm.gdcmt  = GdTbl;			/* Guidance comment table */
  EventPrm.txtobj = OBJ_STRING;		/* Object when text area is touched. */
  EventPrm.sbrobj = NULL;			/* Object when the scroll bar is touched. */
  EventPrm.tchtb  = TchTxtBuf;   	/* Pointer for text scroll bar area	*/
  EventPrm.s_pos  = &ComScr;   		/* Pointer for text and scroll bar information */

  /* Define Screen */
  drawScreen(TRUE);

  /* Define Toucharea */
  initTch();
  LibTxtInit(&EventPrm);			/* Initialization of text input	Display */
  LibTxtTchSet(&EventPrm);			/* Text touch area PUSH	*/
  LibTchStackPush(TchPad);
  LibTchInit();

  if(ip[0] != 0) calcnow = TRUE;

  /* Main loop */
  while (ret_code==0)
  {
    if(LibTxtDsp(&EventPrm))
    {
      LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
      LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
      if (!dispinfo) LibPutDisp();					/* Flush buffer */
    }

    if (dispinfo)
    {
      /* Display calculated values */
      LibStringDsp("Network addr.:", 5, 45, 159, IB_PFONT1);

      ipToString(network, snetwork, IP_IPMASK);
      LibStringDsp(snetwork, 5, 55, 159, IB_PFONT1);

      LibStringDsp("Broadcast addr.:", 5, 67, 159, IB_PFONT1);

	  ipToString(broadcast, sbroadcast, IP_IP);
      LibStringDsp(sbroadcast, 5, 77, 159, IB_PFONT1);

      LibStringDsp("Tot. no of addr.:", 5, 89, 159, IB_PFONT1);

	  sprintf(totaladdr, "%.0f", addresses);
      LibStringDsp(totaladdr, 5, 99, 159, IB_PFONT1);

      LibStringDsp("No. of usable addr.:", 5, 111, 159, IB_PFONT1);

	  sprintf(usableaddr, "%.0f", usableaddresses);
      LibStringDsp(usableaddr, 5, 121, 159, IB_PFONT1);

      LibStringDsp("Prefix length:", 5, 133, 159, IB_PFONT1);

      sprintf(sprefix, "%d", prefix);
      LibStringDsp(sprefix, 5, 143, 159, IB_PFONT1);

      LibPutDisp();
    }

    KeyCode = KEY_NONE;

    if(!calcnow)
    {
	  /* Wait for touch */
      LibTchWait(&tsts);
    } else {
	  tsts.act = ACT_NONE;
	  tsts.x = 0;
	  tsts.y = 0;
	  tsts.ext = 0;
      tsts.obj = OBJ_ENTER;
    }
 
    switch(tsts.obj)
    {
      case OBJ_ENTER:
        /* Check IP and calulate */
        if (LibIconClick(&TIconEnter, &tsts) || calcnow)
        {
            /* Save editor cursor position */
            cursor = EventPrm.cp;
	  		
            /* Convert editor-string to an ip-structure */
        	err = stringToIp(ip, &cip);
			if(err == ERR_NOERROR)
			{
			  /* Redraw screen */
			  drawScreen(FALSE);
			  LibTxtInit(&EventPrm);

			  /* Calculate values */
			  getNetwork(cip, &network);
			  getBroadcast(cip, &broadcast);
			  addresses = totalAddresses(cip);
		      netmaskToNum(cip, &prefix);
			  if (prefix < 32)
			  {
  			    usableaddresses = addresses - 2;
  			  } else {
  			    usableaddresses = addresses;
  			  }

  			  /* Show calculated values */
			  dispinfo = TRUE;
			} else {
			  if (!calcnow)
			  {
			    /* Display error message */
			    errorBox(err, "Input error:");

			    /* Hide calculated values */
			    dispinfo = FALSE;

			    /* Redraw screen */
			    drawScreen(TRUE);
			    initTch();
			    LibTxtInit(&EventPrm);
			    LibTxtTchSet(&EventPrm);
			    LibTchStackPush(TchPad);
			  }
			}

	  		calcnow = FALSE;
			/* Restore editor cursor */
            EventPrm.cp = cursor;
        }
      break;

      case OBJ_CLEAR:
        /* Clear editor area */
        if (LibIconClick(&TIconClear, &tsts))
        {
          ip[0] = 0;
          LibTxtInit(&EventPrm);
          dispinfo = FALSE;
          drawScreen(TRUE);
        }
      break;

      case OBJ_DELETE:
        /* Send delete keycode */
        if (LibIconClick(&TIconDelete, &tsts))
        {
          LibCurBlnkOff();
          KeyCode = KEY_DEL;
        }
      break;

      case OBJ_HIC_ESC:
        /* Exit loop with an ESC */
        ret_code = PV_ESC;
      break;

      case OBJ_INFO:
        /* Display information */
        if (LibIconClick(&TIconInfo, &tsts))
        {
    	  /* Save cursor position */
    	  cursor = EventPrm.cp;

    	  /* Show infobox */
    	  infoBox();

    	  /* Redraw screen */
		  drawScreen(TRUE);
		  initTch();
		  LibTxtInit(&EventPrm);
		  EventPrm.cp = cursor;
		  LibTxtTchSet(&EventPrm);
		  LibTchStackPush(TchPad);
		}
      break;

	  case OBJ_SWITCH:
	  	if (LibIconClick(&TIconSwitch, &tsts))
	  	{
		  ret_code = PV_NET;
	  	}

      default:
        /* Numpad touched */
        if (tsts.obj >= OBJ_NUMPAD0 && tsts.obj <= OBJ_SLASH)
        {
          funcIdx = (tsts.obj & 0xff);
          funcName = KeyPad[funcIdx];
          if (LibIconClick(&TIconKeys[funcIdx], &tsts) == TRUE)
            KeyCode = funcName;
        }
      break;
    }

    /* Send keycode */
    LibTxtInp(KeyCode, &tsts, &EventPrm);
  }
  return(ret_code);
}

/* --------------- ip screen code ---------------- */
void drawScreen(bool push)
{
  int  fontSize;

  LibClrDisp();

  /* Title */
  fontSize = LibGetProStrSize(IB_PFONT3, TITLE);
  LibStringDsp(TITLE, 80 - (fontSize / 2), 2, 159, IB_PFONT3);

  LibGdsLine(0, 15, 159, 15);

  /* Informations */
  LibStringDsp("IP/Subnet:", 5, 19, 159, IB_PFONT1);

  LibPutGraph(1, 1, INFO);
  LibPutGraph(134, 1, NETBTN);

  /* Box around input area */
  LibGdsBox(5, 28, 154, 39);

  /* Keyboard */
  LibPutGraph(KOFFX, KOFFY, KEYBOARD);
  LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);

  if (push) LibPutDisp();
}