/*******************************************************************************/
/*                                                                             */
/*  list.c, part of libList                                                    */
/*  Copyright (C) 2002 Dennis Pries                                            */
/*                                                                             */
/*  This program is free software; you can redistribute it and/or modify       */
/*  it under the terms of the GNU General Public License as published by       */
/*  the Free Software Foundation; either version 2 of the License, or          */
/*  (at your option) any later version.                                        */
/*                                                                             */
/*  This program is distributed in the hope that it will be useful,            */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/*  GNU General Public License for more details.                               */
/*                                                                             */
/*  You should have received a copy of the GNU General Public License          */
/*  along with this program; if not, write to the Free Software                */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */
/*                                                                             */
/*******************************************************************************/

#include	<stdrom.h>
#include	"define.h"
#include	"libc.h"
#include	"l_define.h"
#include	"l_libc.h"
#include 	"list.h"

void initList(int x, int y, int width, int height, char *items[], int numItems, int listID, int scrID, LISTBOX *listbox)
{
	listbox->LISTID = listID;
	listbox->SCRID = scrID;
	listbox->x = x;
	listbox->y = y;
	listbox->width = width;
	listbox->height = height;
	listbox->pos = 0;
	listbox->barpos = 0;
	listbox->items = items;
	listbox->scrollbar.x = x + width - SCR_XSIZE - 1;
	listbox->scrollbar.y = y + 3;
	listbox->scrollbar.size = height - 5;
	listbox->scrollbar.pos = 0;
	listbox->scrollbar.dsp = (height - 3) / 9;
	listbox->scrollbar.vol = numItems;
}

void drawList(LISTBOX *listbox)
{
	LibGdsBox(listbox->x, listbox->y, listbox->x + listbox->width, listbox->y + listbox->height);
	LibGdsLine(listbox->scrollbar.x - 2, listbox->y + 1, listbox->scrollbar.x - 2, listbox->y + listbox->height - 1);
	LibScrollPrint(listbox->scrollbar);
    updateList(listbox);
}

void updateList(LISTBOX *listbox)
{
	int		i;
	int		p;

	LibClrBox(listbox->x + 1, listbox->y + 1, listbox->width - SCR_XSIZE - 4, listbox->height - 2);

	for(i=0; i < listbox->scrollbar.dsp; i++){
		p = listbox->scrollbar.pos + i;
		if (p < listbox->scrollbar.vol)
			LibPutProStr(IB_PFONT1, listbox->x + 3, listbox->y + 3 + i * 9, listbox->items[p], listbox->width - SCR_XSIZE - 5);
	}

	if (listbox->scrollbar.vol > 0) toggleItem(listbox);
	LibPutDisp();
}

int processList(TCHSTS *tsts, LISTBOX *listbox)
{
  int ans;
  int item;

  /* Scrollbar */
  if(tsts->obj == listbox->SCRID)
  {
	ans = LibScrollClick(tsts, &listbox->scrollbar);
	if (ans == 0 &&  tsts->act == ACT_BREAK_IN) /* Up */
	{
		if(listbox->barpos > 0)
		{
			toggleItem(listbox);
			listbox->barpos--;
			toggleItem(listbox);
		} else if (listbox->scrollbar.pos > 0) {
			listbox->scrollbar.pos--;
			LibScrollPrint(listbox->scrollbar);
			updateList(listbox);
		}
	} else if (ans == 1 &&  tsts->act == ACT_BREAK_IN) { /* Down */
		if(listbox->barpos < listbox->scrollbar.dsp - 1)
		{
			toggleItem(listbox);
			listbox->barpos++;
			toggleItem(listbox);
		} else if (listbox->scrollbar.pos < listbox->scrollbar.vol - listbox->scrollbar.dsp - 1) {
			listbox->scrollbar.pos++;
			LibScrollPrint(listbox->scrollbar);
			updateList(listbox);
		}
	} else {
    	updateList(listbox);
	}
  }

  /* Listbox */
  if(tsts->obj == listbox->LISTID && tsts->act == ACT_MAKE)
  {
	item = listGetItem(tsts, listbox);
	if (item < listbox->scrollbar.vol - listbox->scrollbar.pos)
	{
	  toggleItem(listbox);
	  listbox->barpos = item;
	  toggleItem(listbox);
	}
  }

  item = listbox->barpos + listbox->scrollbar.pos;
  return(item);
}

int listGetItem(TCHSTS *tsts, LISTBOX *listbox)
{
	int item;
	item = (tsts->y - 2 - listbox->y) / 9;
	if (item >= listbox->scrollbar.dsp) item = listbox->scrollbar.dsp - 1;
 	return(item);
}

void toggleItem(LISTBOX *listbox)
{
	LibReverse(		listbox->x + 2, listbox->y + 2 + listbox->barpos * 9, listbox->width - SCR_XSIZE - 6, 9);
	LibPutDispBox(	listbox->x + 2, listbox->y + 2 + listbox->barpos * 9, listbox->width - SCR_XSIZE - 6, 9);
}

void listInitTch(LISTBOX *listbox)
{
	/* Add Listbox to Touchstack*/
	listbox->Tch[0].x1 = listbox->x;
	listbox->Tch[0].y1 = listbox->y;
	listbox->Tch[0].x2 = listbox->x + listbox->width - SCR_XSIZE - 6;
	listbox->Tch[0].y2 = listbox->y + listbox->height;
	listbox->Tch[0].act = ACT_ICON;
	listbox->Tch[0].obj = listbox->LISTID;
	listbox->Tch[0].ext = 0x0000;

	/* Add Scrollbar to Touchstack*/
	listbox->Tch[1].x1 = listbox->scrollbar.x;
	listbox->Tch[1].y1 = listbox->scrollbar.y;
	listbox->Tch[1].x2 = listbox->scrollbar.x + SCR_XSIZE - 1;
	listbox->Tch[1].y2 = listbox->scrollbar.y + listbox->scrollbar.size - 1;
	listbox->Tch[1].act = ACT_SCR_BAR;
	listbox->Tch[1].obj = listbox->SCRID;
	listbox->Tch[1].ext = 0x0000;

	listbox->Tch[2].x1 = 0;
	listbox->Tch[2].y1 = 0;
	listbox->Tch[2].x2 = 0;
	listbox->Tch[2].y2 = 0;
	listbox->Tch[2].act = ACT_NONE;
	listbox->Tch[2].obj = OBJ_END;
	listbox->Tch[2].ext = 0x0000;

	LibTchStackPush(listbox->Tch);
}

void tchLast()
{
	TCHTBL Tch[1] = {0, 0, 0, 0, ACT_NONE, OBJ_END, 0x0000};
	LibTchStackPush(Tch);
}