/*******************************************************************************/
/*                                                                             */
/*  netloop.c, network calculations main loop                                  */
/*  Copyright (C) 2002 Dennis Pries                                            */
/*                                                                             */
/*  This program is free software; you can redistribute it and/or modify       */
/*  it under the terms of the GNU General Public License as published by       */
/*  the Free Software Foundation; either version 2 of the License, or          */
/*  (at your option) any later version.                                        */
/*                                                                             */
/*  This program is distributed in the hope that it will be useful,            */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/*  GNU General Public License for more details.                               */
/*                                                                             */
/*  You should have received a copy of the GNU General Public License          */
/*  along with this program; if not, write to the Free Software                */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */
/*                                                                             */
/*******************************************************************************/

#include	"pvipcalc.h"
#include	"ip.h"
#include	"gdata.h"
#include	"generic.h"
#include	"loops.h"
#include	"list.h"

int counter;
int oldcounter;
bool resetCounter = TRUE;

int netloop(char *base, char *ip, char *otherip, int *barpos, int *scrbarpos)
{
  int ret_code = 0;
  int i;
  int err;
  int cursor;

  int maxNetworks;
  int item;
  byte net1, net2;
  bool forceRedraw = TRUE;
  bool calcnow = FALSE;

  TCHSTS tsts;
  char *listItems[MAX_LISTITEMS];
  char listData[MAX_LISTITEMS][21];
  char *ptr;
  char tmp[32];
  char temp[32];

  TXTP		BaseNet;
  TXTP		IPNet;
  TXTP		*ActiveTxt;
  word		GdTbl[1];
  TCHTBL	TchTxtBuf1[3];
  T_SCR_POS	ComScr1;
  TCHTBL	TchTxtBuf2[3];
  T_SCR_POS	ComScr2;
  LISTBOX	listbox;

  IP		ipBaseNet;
  IP		ipNet;
  IP        tmpip;

  int		funcIdx;
  char      funcName;
  char		KeyCode;

  float	    total;
  float     totalNets = 0.0;

  GdTbl[0]		  = 0xffff;		/* No Guidance comment */
  BaseNet.st_x   = 7;			/* Start coordinate (X) of text display	*/
  BaseNet.st_y   = 30;			/* Start coordinate (Y) of text display	*/
  BaseNet.ed_x   = 79;			/* End coordinate (X) of text display */
  BaseNet.it_y   = 9;			/* Text display line spacing (Y) */
  BaseNet.MAXGYO = 1;			/* Number of text display lines	*/
  BaseNet.font   = IB_PFONT1;	/* Display font type */
  BaseNet.csen   = TRUE;		/* Cursor display enabled (Keyword registration enabled) */
  BaseNet.rtnen  = 0x02;		/* CR code display disabled	*/
  BaseNet.maxmj  = 15;			/* Maximum number of allowable input characters. */
  BaseNet.txbf   = base;   		/* Designation of text buffer address */
  BaseNet.gdcmt  = GdTbl;		/* Guidance comment table */
  BaseNet.txtobj = OBJ_NETTXT;	/* Object when text area is touched. */
  BaseNet.sbrobj = NULL;		/* Object when the scroll bar is touched. */
  BaseNet.tchtb  = TchTxtBuf1; 	/* Pointer for text scroll bar area	*/
  BaseNet.s_pos  = &ComScr1;   	/* Pointer for text and scroll bar information */

  IPNet.st_x   = 7;				/* Start coordinate (X) of text display	*/
  IPNet.st_y   = 54;			/* Start coordinate (Y) of text display	*/
  IPNet.ed_x   = 152;			/* End coordinate (X) of text display */
  IPNet.it_y   = 9;			    /* Text display line spacing (Y) */
  IPNet.MAXGYO = 1;				/* Number of text display lines	*/
  IPNet.font   = IB_PFONT1;		/* Display font type */
  IPNet.csen   = TRUE;			/* Cursor display enabled (Keyword registration enabled) */
  IPNet.rtnen  = 0x02;			/* CR code display disabled	*/
  IPNet.maxmj  = 31;			/* Maximum number of allowable input characters. */
  IPNet.txbf   = ip;   			/* Designation of text buffer address */
  IPNet.gdcmt  = GdTbl;			/* Guidance comment table */
  IPNet.txtobj = OBJ_IPTXT;		/* Object when text area is touched. */
  IPNet.sbrobj = NULL;			/* Object when the scroll bar is touched. */
  IPNet.tchtb  = TchTxtBuf2;   	/* Pointer for text scroll bar area	*/
  IPNet.s_pos  = &ComScr2;     	/* Pointer for text and scroll bar information */

  /* init data pointer */
  for(i=0; i<MAX_LISTITEMS; i++)
  {
    listItems[i] = &listData[i][0];
  }

  initList(5, 74, 102, 84, listItems, 0, OBJ_LIST, OBJ_SCRBAR, &listbox);

  drawNetScreen(TRUE, &listbox);

  ActiveTxt = &BaseNet;
  LibTxtInit(&IPNet);
  LibTxtDsp(&IPNet);
  LibTxtInit(&BaseNet);
  LibTxtDsp(&BaseNet);

  initTch();
  LibTxtTchSet(&BaseNet);
  LibTxtTchSet(&IPNet);
  LibTchStackPush(TchPad);
  LibTchStackPush(ClassBtns);
  listInitTch(&listbox);
  LibTchInit();

  if((ip[0] != 0) && (base[0] != 0)) calcnow = TRUE;

  while(ret_code==0)
  {
	if (LibTxtDsp(ActiveTxt) || forceRedraw)
	{
      LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
      LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
      LibGdsClr(5, 65, 159, 74);
	  LibPutDisp();

      if (totalNets > 0.0)
      {
	    sprintf(tmp, "%.0f subnets", totalNets);
        LibStringDsp(tmp, 5, 65, 159, IB_PFONT1);
      }

      drawList(&listbox);
      forceRedraw = FALSE;
    }

    KeyCode = KEY_NONE;

    if (!calcnow)
    {
      LibTchWait(&tsts);
    } else {
	  tsts.act = ACT_NONE;
	  tsts.x = 0;
	  tsts.y = 0;
	  tsts.ext = 0;
      tsts.obj = OBJ_ENTER;
    }

	switch(tsts.obj)
    {
      case OBJ_ENTER:
        /* Check IP and calulate */
        if (LibIconClick(&TIconEnter, &tsts) || calcnow)
        {
			strcpy(tmp, "0.0.0.0/");
			strcpy(temp, base);
			strcat(tmp, temp);
			strcpy(listItems[0], tmp);
			strcpy(listItems[1], base);

			err = stringToIp(tmp, &ipBaseNet);
			if (err == ERR_NOERROR)
			{
			  err = stringToIp(ip, &ipNet);
			  if (err == ERR_NOERROR)
			  {
			    /* Get base ip adress */
			    for(i=0; i<4; i++)
 			      ipBaseNet.ip[i] = ipNet.ip[i];

 			    getNetwork(ipBaseNet, &tmpip);

			    for(i=0; i<4; i++)
 			      ipBaseNet.ip[i] = tmpip.ip[i];

				netmaskToNum(ipBaseNet, &net1);
				netmaskToNum(ipNet, &net2);

				if (net1 < net2 && net2 < 32)
				{
  			      if (resetCounter)
			      {
			        counter = 0;
			        totalNets = totalNetworks(ipBaseNet, ipNet);
			        if (totalNets > 65535)
			        {
			         maxNetworks = 65535;
			        } else {
			         maxNetworks = (int) totalNets;
			        }

  			        resetCounter = FALSE;
			        sprintf(tmp, "%.0f subnets", totalNets);
                    LibGdsClr(5, 65, 159, 74);
                    LibStringDsp(tmp, 5, 65, 159, IB_PFONT1);
			      }

				  if(calcnow) counter = oldcounter;

				  oldcounter = counter;

			      total = 0.0;
			      for(i=counter; i < (MAX_LISTITEMS + counter); i++)
			      {
			        if (i == maxNetworks)
			        {
			          resetCounter = TRUE;
			          break;
			        }
			        total = getNetworkN(ipBaseNet, ipNet, &tmpip, (float) i, total);
			        ipToString(tmpip, tmp, IP_IP);
		            sprintf(listItems[i - counter], "%d. %s", i, tmp);
		          }

				  if (!calcnow)
				  {
			        listbox.barpos = 0;
			        listbox.scrollbar.pos = 0;
			      } else {
			        listbox.barpos = *barpos;
			        listbox.scrollbar.pos = *scrbarpos;
			      }
			      listbox.scrollbar.vol = i - counter;
			      drawList(&listbox);

   		   	      counter = counter + (i - counter);
   			    } else {
      		      if (!calcnow)
      		      {
      		        errorBox(ERR_NETMASKINVALID, "Input error:");

  	      	        /* Redraw screen */
		  	        drawNetScreen(TRUE, &listbox);
          	        initTch();
          	        LibTxtInit(&BaseNet);
           	        LibTxtInit(&IPNet);
    	            LibTxtDsp(&BaseNet);
          	        LibTxtDsp(&IPNet);
                    LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
                    LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
                    LibPutDisp();
          	        LibTxtTchSet(&BaseNet);
          	        LibTxtTchSet(&IPNet);
          	        LibTchStackPush(TchPad);
          	        LibTchStackPush(ClassBtns);
          	        listInitTch(&listbox);
          	        LibTchInit();
                  }
          	      forceRedraw = TRUE;
			    }
			  } else {
    		    if (!calcnow)
    		    {
    		      errorBox(err, "IP/Subnet:");

	      	      /* Redraw screen */
		  	      drawNetScreen(TRUE, &listbox);
          	      initTch();
          	      LibTxtInit(&BaseNet);
           	      LibTxtInit(&IPNet);
    	          LibTxtDsp(&BaseNet);
          	      LibTxtDsp(&IPNet);
                  LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
                  LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
                  LibPutDisp();
          	      LibTxtTchSet(&BaseNet);
          	      LibTxtTchSet(&IPNet);
          	      LibTchStackPush(TchPad);
          	      LibTchStackPush(ClassBtns);
          	      listInitTch(&listbox);
          	      LibTchInit();
          	    }
          	    forceRedraw = TRUE;
			  }
			} else {
			  if (!calcnow)
			  {
			    errorBox(err, "Base Mask:");

	      	    /* Redraw screen */
		  	    drawNetScreen(TRUE, &listbox);
          	    initTch();
          	    LibTxtInit(&BaseNet);
          	    LibTxtInit(&IPNet);
    	        LibTxtDsp(&BaseNet);
          	    LibTxtDsp(&IPNet);
                LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
                LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
                LibPutDisp();
          	    LibTxtTchSet(&BaseNet);
          	    LibTxtTchSet(&IPNet);
          	    LibTchStackPush(TchPad);
          	    LibTchStackPush(ClassBtns);
          	    listInitTch(&listbox);
          	    LibTchInit();
          	  }
          	  forceRedraw = TRUE;
			}
			calcnow = FALSE;
		}
	  break;

	  case OBJ_SCRBAR:
	  	processList(&tsts, &listbox);
	  break;

	  case OBJ_LIST:
	    item = processList(&tsts, &listbox);

		if(tsts.act == ACT_MAKE)
		{
		  if (listbox.scrollbar.vol > 0)
          {
            ptr = strstr(listData[item], " ");
            if (ptr != NULL)
            {
              ptr++;
		      strcpy(otherip, ptr);
		      ptr = strstr(ip, "/");
		      if (ptr != NULL)
		      {
		        strcat(otherip, ptr);
		        *barpos = listbox.barpos;
		        *scrbarpos = listbox.scrollbar.pos;
		        ret_code = PV_IP;
		      }
		    }
          }
    	}
	  break;

      case OBJ_HIC_ESC:
        /* Exit loop with an ESC */
        ret_code = PV_ESC;
      break;

      case OBJ_INFO:
        /* Display information */
        if (LibIconClick(&TIconInfo, &tsts))
        {
    	  /* Save cursor position */
    	  cursor = ActiveTxt->cp;

    	  /* Show infobox */
    	  infoBox();

    	  /* Redraw screen */
		  drawNetScreen(TRUE, &listbox);
          initTch();
          LibTxtInit(ActiveTxt);
          ActiveTxt->cp = cursor;
          LibTxtTchSet(&BaseNet);
          LibTxtTchSet(&IPNet);
          LibTchStackPush(TchPad);
          LibTchStackPush(ClassBtns);
          listInitTch(&listbox);
          LibTchInit();
          forceRedraw = TRUE;
		}
      break;

	  case OBJ_IPTXT:
	    if (ActiveTxt != &IPNet)
	    {
	      LibTxtInit(ActiveTxt);
	      LibTxtDsp(ActiveTxt);
          LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
          LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
     	  LibPutDisp();
	      ActiveTxt = &IPNet;
	      LibTxtInit(ActiveTxt);
	    }
	  break;

	  case OBJ_NETTXT:
	    if (ActiveTxt != &BaseNet)
	    {
	      LibTxtInit(ActiveTxt);
	      LibTxtDsp(ActiveTxt);
          LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
          LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
     	  LibPutDisp();
	      ActiveTxt = &BaseNet;
	      LibTxtInit(ActiveTxt);
	    }
	  break;

      case OBJ_CLEAR:
        /* Clear editor area */
        if (LibIconClick(&TIconClear, &tsts))
        {
          ActiveTxt->txbf[0] = 0;
	      LibTxtInit(ActiveTxt);
          LibGdsClr(ActiveTxt->st_x,
                    ActiveTxt->st_y,
                    ActiveTxt->ed_x,
                    ActiveTxt->st_y + ActiveTxt->it_y - 1);
          forceRedraw = TRUE;
          resetCounter = TRUE;
	      listbox.scrollbar.vol = 0;
        }
      break;

      case OBJ_DELETE:
        /* Send delete keycode */
        if (LibIconClick(&TIconDelete, &tsts))
        {
          LibCurBlnkOff();
          KeyCode = KEY_DEL;
          resetCounter = TRUE;
	      listbox.scrollbar.vol = 0;
        }
      break;

	  case OBJ_SWITCH:
	  	if (LibIconClick(&TIconSwitch, &tsts))
	  	{
		  ret_code = PV_IP;
	  	}
	  break;

      case OBJ_ABTN:
	  	if (LibIconClick(&TIconABtn, &tsts))
	  	{
		  strcpy(BaseNet.txbf, "255.0.0.0");
		  LibTxtInit(&BaseNet);
		  LibTxtDsp(&BaseNet);
          LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
          LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
     	  LibPutDisp();
	      ActiveTxt = &IPNet;
	      LibTxtInit(ActiveTxt);
		  forceRedraw = TRUE;
		  resetCounter = TRUE;
	      listbox.scrollbar.vol = 0;
	  	}
      break;

      case OBJ_BBTN:
	  	if (LibIconClick(&TIconBBtn, &tsts))
	  	{
		  strcpy(BaseNet.txbf, "255.255.0.0");
		  LibTxtInit(&BaseNet);
		  LibTxtDsp(&BaseNet);
          LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
          LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
     	  LibPutDisp();
	      ActiveTxt = &IPNet;
	      LibTxtInit(ActiveTxt);
		  forceRedraw = TRUE;
		  resetCounter = TRUE;
	      listbox.scrollbar.vol = 0;
	  	}
      break;

      case OBJ_CBTN:
	  	if (LibIconClick(&TIconCBtn, &tsts))
	  	{
		  strcpy(BaseNet.txbf, "255.255.255.0");
		  LibTxtInit(&BaseNet);
		  LibTxtDsp(&BaseNet);
          LibPutGraph(KOFFX, KOFFY, KEYBOARD);			/* Draw Keyboard */
          LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);
     	  LibPutDisp();
	      ActiveTxt = &IPNet;
	      LibTxtInit(ActiveTxt);
		  forceRedraw = TRUE;
		  resetCounter = TRUE;
	      listbox.scrollbar.vol = 0;
	  	}
      break;

      default:
        /* Numpad touched */
        if (tsts.obj >= OBJ_NUMPAD0 && tsts.obj <= OBJ_SLASH)
        {
          funcIdx = (tsts.obj & 0xff);
          funcName = KeyPad[funcIdx];
          if (LibIconClick(&TIconKeys[funcIdx], &tsts) == TRUE)
          {
            KeyCode = funcName;
            resetCounter = TRUE;
    	    listbox.scrollbar.vol = 0;
          }
        }
      break;
    }

    /* Send keycode */
    LibTxtInp(KeyCode, &tsts, ActiveTxt);
  }

  return(ret_code);
}

/* ----------------------- net screen code ---------------------- */
void drawNetScreen(bool push, LISTBOX *listbox)
{
  int  fontSize;

  LibClrDisp();

  /* Title */
  fontSize = LibGetProStrSize(IB_PFONT3, TITLE);
  LibStringDsp(TITLE, 80 - (fontSize / 2), 2, 159, IB_PFONT3);

  LibGdsLine(0, 15, 159, 15);

  /* Informations */
  LibStringDsp("Base Netmask:", 5, 19, 159, IB_PFONT1);
  LibStringDsp("IP/Subnet:", 5, 43, 159, IB_PFONT1);

  LibPutGraph(1, 1, INFO);
  LibPutGraph(134, 1, IPBTN);

  LibStringDsp("Class:", 112, 19, 159, IB_PFONT1);
  LibPutGraph(112, 28, ABTN);
  LibPutGraph(127, 28, BBTN);
  LibPutGraph(142, 28, CBTN);

  /* Box around input area */
  LibGdsBox(5, 28, 81, 39);
  LibGdsBox(5, 52, 154, 63);

  /* Keyboard */
  LibPutGraph(KOFFX, KOFFY, KEYBOARD);
  LibGdsClr(0, KOFFY, KOFFX - 1, KOFFY + 71);

  if (push) LibPutDisp();
}